/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Navigator.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomNavigator.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomNavigator.h"
#include "gdom/PrivateIntGdomNavigator.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::Navigator* obj)
{
    return WebCore::cacheDOMObject<WebCore::Navigator, GDOMNavigator>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMNavigator::GDOMNavigator(PassRefPtr<WebCore::Navigator> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMNavigator::~GDOMNavigator()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

gboolean
gdom_navigator_java_enabled (GdomNavigator *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Navigator * item = (gdom_navigator_get_private(thiz));
    g_return_val_if_fail (item, 0);
    gboolean res = item->javaEnabled();
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "Navigator.h"
#include <gdom/GdomNavigator.h>
#include <gdom/PrivateGdomNavigator.h>
#include <gdom/PrivateIntGdomNavigator.h>
#include <gdom/GdomNavigatorprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomNavigator, gdom_navigator, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomNavigatorPrivate {
    class WebCore::Navigator *item;
};

#define GDOM_NAVIGATOR_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_NAVIGATOR, GdomNavigatorPrivate))
*/
G_END_DECLS
WebCore::Navigator *
gdom_navigator_get_private(GdomNavigator *request)
{
    GDOMNavigator *res;
    res = static_cast<GDOMNavigator*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::Navigator*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_APP_CODE_NAME,
    PROP_APP_NAME,
    PROP_LANGUAGE,
    PROP_USER_AGENT,
    PROP_PLATFORM,
    PROP_PLUGINS,
    PROP_MIME_TYPES,
    PROP_PRODUCT,
    PROP_PRODUCT_SUB,
    PROP_VENDOR,
    PROP_VENDOR_SUB,
    PROP_COOKIE_ENABLED,
    PROP_ON_LINE,
};


static void gdom_navigator_finalize(GObject* object)
{
    /*GdomNavigator* request = GDOM_NAVIGATOR(object);*/
    /*GdomNavigatorPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_navigator_parent_class)->finalize(object);
}

static void gdom_navigator__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomNavigator *self = GDOM_NAVIGATOR(object);
    WebCore::Navigator * gself = (gdom_navigator_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_navigator__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomNavigator *self = GDOM_NAVIGATOR(object);
    WebCore::Navigator * gself = (gdom_navigator_get_private(self));
    switch (prop_id) {
    case PROP_APP_CODE_NAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->appCodeName()));
         break;
    }
    case PROP_APP_NAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->appName()));
         break;
    }
    case PROP_LANGUAGE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->language()));
         break;
    }
    case PROP_USER_AGENT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->userAgent()));
         break;
    }
    case PROP_PLATFORM:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->platform()));
         break;
    }
    case PROP_PLUGINS:
    {
         PassRefPtr<WebCore::PluginArray> ptr = gself->plugins();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_MIME_TYPES:
    {
         PassRefPtr<WebCore::MimeTypeArray> ptr = gself->mimeTypes();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_PRODUCT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->product()));
         break;
    }
    case PROP_PRODUCT_SUB:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->productSub()));
         break;
    }
    case PROP_VENDOR:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->vendor()));
         break;
    }
    case PROP_VENDOR_SUB:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->vendorSub()));
         break;
    }
    case PROP_COOKIE_ENABLED:
    {
         g_value_set_boolean (value, gself->cookieEnabled());
         break;
    }
    case PROP_ON_LINE:
    {
         g_value_set_boolean (value, gself->onLine());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_navigator_class_init(GdomNavigatorClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_navigator_finalize;
    g_klass->set_property = gdom_navigator__set_property_fn;
    g_klass->get_property = gdom_navigator__get_property_fn;

    g_object_class_install_property (g_klass, PROP_APP_CODE_NAME,
            g_param_spec_string ("app_code_name", /* name */
                "navigator_app_code_name", /* short description */
                "read-only  gchar * Navigator.app_code_name", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_APP_NAME,
            g_param_spec_string ("app_name", /* name */
                "navigator_app_name", /* short description */
                "read-only  gchar * Navigator.app_name", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_LANGUAGE,
            g_param_spec_string ("language", /* name */
                "navigator_language", /* short description */
                "read-only  gchar * Navigator.language", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_USER_AGENT,
            g_param_spec_string ("user_agent", /* name */
                "navigator_user_agent", /* short description */
                "read-only  gchar * Navigator.user_agent", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PLATFORM,
            g_param_spec_string ("platform", /* name */
                "navigator_platform", /* short description */
                "read-only  gchar * Navigator.platform", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PLUGINS,
            g_param_spec_object ("plugins", /* name */
                "navigator_plugins", /* short description */
                "read-only  GdomPluginArray * Navigator.plugins", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_PLUGIN_ARRAY, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_MIME_TYPES,
            g_param_spec_object ("mime_types", /* name */
                "navigator_mime_types", /* short description */
                "read-only  GdomMimeTypeArray * Navigator.mime_types", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_MIME_TYPE_ARRAY, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PRODUCT,
            g_param_spec_string ("product", /* name */
                "navigator_product", /* short description */
                "read-only  gchar * Navigator.product", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PRODUCT_SUB,
            g_param_spec_string ("product_sub", /* name */
                "navigator_product_sub", /* short description */
                "read-only  gchar * Navigator.product_sub", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_VENDOR,
            g_param_spec_string ("vendor", /* name */
                "navigator_vendor", /* short description */
                "read-only  gchar * Navigator.vendor", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_VENDOR_SUB,
            g_param_spec_string ("vendor_sub", /* name */
                "navigator_vendor_sub", /* short description */
                "read-only  gchar * Navigator.vendor_sub", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_COOKIE_ENABLED,
            g_param_spec_boolean ("cookie_enabled", /* name */
                "navigator_cookie_enabled", /* short description */
                "read-only  gboolean Navigator.cookie_enabled", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_ON_LINE,
            g_param_spec_boolean ("on_line", /* name */
                "navigator_on_line", /* short description */
                "read-only  gboolean Navigator.on_line", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomNavigatorPrivate));*/
}

static void gdom_navigator_init(GdomNavigator* request)
{
    /*GdomNavigatorPrivate* priv = GDOM_NAVIGATOR_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

